#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

/* xread:
 *   Reads exactly <size> bytes, retrying until enough bytes have been read.
 *   Return: number of bytes read (> 0) on success
 *           0 on eof (nothing more to read)
 *           -1 on error
 */
int
xread(fd, buf, size)
  int fd;
  void *buf;
  size_t size;
{
	int c;
	size_t left;

	left = size;
	while (left > 0) {
		c = read(fd, buf, left);

		/* returns -1 on error, 0 on eof */
		if (c <= 0)
			return c;

		left -= c;
		buf = (void *) buf + c;
	}

	return size;
}

/* xwrite:
 *   Writes exactly <size> bytes, retrying until enough bytes have been read.
 *   Return: number of bytes read (> 0) on success
 *           0 on eof (nothing more to read)
 */
int
xwrite(fd, buf, size)
  int fd;
  void *buf;
  size_t size;
{
	int c;
	size_t left;

	left = size;
	while (left > 0) {
		c = write(fd, buf, left);

		/* returns -1 on error (0 is not relevant with write) */
		if (c == -1)
			return c;

		left -= c;
		buf = (void *) buf + c;
	}

	return size;
}

/* timeval_subtract:
 *   Subtract the `struct timeval' values X and Y, storing the result in Y.
 *   (Inspired by the one found in GNU libc documentation.)
 */
void
timeval_subtract2 (early, late)
  struct timeval *early, *late;
{
	if (late->tv_usec < early->tv_usec) {
		late->tv_sec--;
		late->tv_usec = 1000000+late->tv_usec;
	}
	late->tv_sec -= early->tv_sec;
	late->tv_usec -= early->tv_usec;
}

/* parse_size:
 *   Like atoi but also takes care of 'k' (for 1024-multiple) and 'm'
 *   (1024*1024-multiple) at end of string.
 */
long
parse_size (str)
  char *str;
{
	int multiple = 1;

	/* remove unnecessary characters in str */
	int ipos, cpos;
	ipos = 0;
	for(cpos = 0 ; str[cpos] ; cpos++) {
		if (!isblank(str[cpos]))
			str[ipos++] = str[cpos];
	}
	str[ipos] = 0;

	/* check last character of string */
	ipos--;
	if (ipos) {
		if (str[ipos] == 'k') {
			str[ipos] = 0;
			multiple = 1024;
		} else if (str[ipos] == 'M') {
			str[ipos] = 0;
			multiple = 1024*1024;
		}
	}

	return atoi(str)*multiple;
}
